from epyk.core.Page import Report
rptObj = Report()




rptObj.auth.facebook_sign_in('1553869208115832', insert_button=True, debug=True)
print(rptObj.outs.html_file(path=r"C:\Users\nelso\Desktop", name='fb_signin'))




import os
import sys
import logging

import collections
import subprocess

from epyk.core import Page
from epyk.core.js import Imports


class App(object):

  def __init__(self, app_path, app_name, alias, name, report=None):
    self.imports = collections.OrderedDict({"http": 'http', 'url': 'url', 'fs': 'fs'})
    self.import_launchers = []
    self.vars, self.__map_var_names, self._report = {}, {}, report
    self._app_path, self._app_name = app_path, app_name
    self.alias, self.__path, self.className, self.__components = alias, 'apps', name, None
    self.comps = {}

  def require(self, module, alias=None, launcher=""):
    """
    Description:
    ------------
    Add external modules to the application

    Related Pages:

      https://www.w3schools.com/nodejs/nodejs_modules.asp

    Attributes:
    ----------
    :param module: String. The module name.
    :param alias: String. Optional. The alias name for the JavaScript Variable name
    :param launcher: String. Optional. The JavasCript String to be added to the page just after the imports
    """
    self.imports[module] = alias or module
    if launcher:
      self.import_launchers.append(launcher)

  @property
  def name(self):
    """
    Description:
    ------------
    Return the prefix of the component module (without any extension)
    """
    return self.className

  @property
  def path(self):
    """
    Description:
    ------------
    Return the full path of the component modules
    """
    return os.path.join("./", self.__path, self.name).replace("\\", "/")

  def checkImports(self, app_path=None):
    """
    Description:
    ------------
    Check if the npm modules are defined on the server for the defined application.

    Attributes:
    ----------
    :param app_path: String. Optional. the Path of the NodeJs application
    """
    app_path = app_path or self._app_path
    node_modules = os.path.join(app_path, "node_modules")
    self.imports = {"showdown": "showdown"}
    for imp, alias in self.imports.items():
      if imp in Imports.JS_IMPORTS:
        if 'register' in Imports.JS_IMPORTS[imp]:
          if not os.path.exists(os.path.join(node_modules, Imports.JS_IMPORTS[imp]['register'].get('npm', imp))):
            logging.warning("Missing module - %s - on the nodeJsServer" % imp)

  def export(self, path=None, target_path=None):
    """
    Description:
    ------------
    Export the NodeJs application

    Attributes:
    ----------
    :param path: String. The NodeJs server path
    :param target_path: String. The folder location on the server. for example ['src', 'app']
    """
    self.__path = path or self.__path
    if target_path is None:
      target_path = []
    target_path.append(self.__path)
    module_path = os.path.join(self._app_path, self._app_name, *target_path)
    if not os.path.exists(module_path):
      os.makedirs(module_path)
    self._report.outs.html_file(path=module_path, name=self.name)
    requirements = []
    for imp, alias in self.imports.items():
      if imp in Imports.JS_IMPORTS:
        if 'register' in Imports.JS_IMPORTS[imp]:
          imp = Imports.JS_IMPORTS[imp]['register'].get('npm', imp)
          alias = Imports.JS_IMPORTS[imp]['register'].get('alias', alias)
      requirements.append("var %s = require('%s')" % (alias, imp))
    js_reqs = ";\n".join(requirements)

    # Write the JavaScript launcher
    with open(os.path.join(module_path, "%s.js" % self.name), "w") as f:
      f.write('''
%s;

fs.readFile('./%s.html', function (err, html) {
    if (err) {throw err;}       
    http.createServer(function(request, response) {  
        response.writeHeader(200, {"Content-Type": "text/html"});  
        response.write(html);  
        response.end();  
    }).listen(8000);
}); ''' % (js_reqs, self.name))


class Node(object):

  def __init__(self, app_path, name=None):
    self._app_path, self._app_name = app_path, name
    self._route, self._fmw_modules = None, None
    self._page = None

  def npm(self, packages):
    """
    Description:
    ------------
    Use npm install on a package.
    Can be done directly on the nodeJs app using the command line:
      npm install package1 package2 .....

    Attributes:
    ----------
    :param packages: List. The list of packages to install retrieved from requirements()
    """
    subprocess.run('npm install %s --save' % " ".join(packages), shell=True, cwd=self._app_path)
    print("%s packages installed" % len(packages))

  def ls(self):
    """
    Description:
    ------------
    Search the registry for packages matching terms
    """
    subprocess.run('npm ls', shell=True, cwd=self._app_path)

  def run(self, name):
    """
    Description:
    ------------
    The file you have just created must be initiated by Node.js before any action can take place.

    Related Pages:

      https://www.w3schools.com/nodejs/nodejs_get_started.asp

    Attributes:
    ----------
    :param name: String. The script name
    """
    subprocess.run('node %s' % name, shell=True, cwd=self._app_path)

  def docs(self, package):
    """
    Description:
    ------------
    Display the README.md / documentation / npmjs.org page of a give library

    Attributes:
    ----------
    :param package: String. The package alias
    """
    subprocess.run('npm docs %s' % package, shell=True, cwd=self._app_path)

  def update(self, packages):
    """
    Description:
    ------------
    Update all the packages listed to the latest version (specified by the tag config). Also install missing packages

    Attributes:
    ----------
    :param packages: List. The list of packages to be installed
    """
    subprocess.run('npm update %s' % " ".join(packages), shell=True, cwd=self._app_path)
    print("%s packages updated" % len(packages))

  def uninstall(self, packages):
    """
    Description:
    ------------
    Uninstall a package, completely removing everything npm installed on its behalf

    Attributes:
    ----------
    :param packages: List. The list of packages to be installed
    """
    subprocess.run('npm uninstall %s' % " ".join(packages), shell=True, cwd=self._app_path)
    print("%s packages uninstalled" % len(packages))

  def install(self):
    """
    Description:
    ------------
    Install or update Vue.Js on the defined path
    """
    subprocess.run('npm install -g @vue/cli', shell=True, cwd=self._app_path)
    print("Angular CLI installed")

  def page(self, selector=None, name=None, report=None, auto_route=False):
    """
    Description:
    ------------
    Create a specific Application as a component in the Angular framework.

    Unlike a basic component, the application will be routed to be accessed directly.

    Description:
    ------------
    :param report: Object. A report object
    :param selector: String. The url route for this report in the Angular app
    :param name: String. The component classname in the Angular framework
    """
    if name is None:
      script = os.path.split(sys._getframe().f_back.f_code.co_filename)[1][:-3]
      if selector is None:
        selector = script.replace("_", "-")
    report = report or Page.Report()
    report.framework("NODE")
    self._page = App(self._app_path, self._app_name, selector, name, report=report)
    #if auto_route:
    #  self.route.add(self.__page.className, self.__page.alias, self.__page.path)
    return self._page

  def publish(self, app_name=None, target_path=None):
    """
    Description:
    ------------
    Publishh the Vue.js application

    Attributes:
    ----------
    :param app_name:
    :param target_path: List  for example ['src', 'app']
    """
    if self._page is not None:
      self._page.export(target_path=target_path)




from epyk.core.html import tables as html_tables


class Tabulators(object):
  def __init__(self, context):
    self.parent = context

  def table(self, records=None, cols=None, rows=None, width=(100, '%'), height=(None, 'px'), htmlCode=None, options=None, profile=None):
    """

    :param records:
    :param cols:
    :param rows:
    :param width:
    :param height:
    :param htmlCode:
    :param options:
    :param profile:
    """
    cols = cols or []
    rows = rows or []
    if records is not None and not cols and not rows:
      cols = list(records[0].keys())

    table_options_dflts = {'selectable': False, 'index': '_row', 'layout': 'fitColumns', 'pagination': 'local',
                           'paginationSize': 25, 'resizableRows': False, 'movableColumns': True}
    if options is not None:
      table_options_dflts.update(options)

    table = html_tables.HtmlTableTabulator.Table(self.parent.context.rptObj, records, width, height, htmlCode, options, profile)
    for c in cols + rows:
      table.add_column(c)
      #col_def.cssClass.center() # = "dt-center, dt-test"
    return table

  def heatmap(self):
    pass

  def intensity(self):
    pass

  def hierarchy(self):
    pass



from epyk.core.js.packages import packageImport
from epyk.core.js.fncs import JsFncs

class Auth(object):


    def __init__(self, report):
        self._report = report

    @packageImport('google-platform')
    def google_sign_in(self, client_id, scopes=['profile', 'email'], insert_button=True, debug=False):
        """
        Description:
        -------------
        Allows to add a google signin capability to the page
        This needs a working server to work

        Related Pages:

      https://developers.google.com/identity/sign-in/web/sign-in

        :param client_id: the google webapp client id defined here: https://console.developers.google.com/apis/credentials
        :param insert_button: choose whether the button will be inserted when this function is called or not
        :return: Epyk Obj
        """

        headers = self._report.headers
        headers.meta.custom('google-signin-scope', ' '.join(scopes))
        headers.meta.custom('google-signin-client_id', client_id)
        g_button = self._report.ui.div()
        g_button.style.no_class()
        g_button.style.clear_style()
        g_button.style.add_classes.external('g-signin2')
        g_button.attr.update({'data-onsuccess': 'onSignIn'})

        fnc_list = [self._report.js.object('profile', '''profile = googleUser.getBasicProfile()'''),
                    self._report.js.object('id_token', '''id_token = googleUser.getAuthResponse().id_token''')]
        if debug:
            fnc_list.append(self._report.js.console.log(self._report.js.getVar('id_token', 'object'), skip_data_convert=True))
        fnc_list.append(self._report.js.return_(self._report.js.getVar('id_token', 'object')))
        g_button.options.managed = insert_button
        self._report.js.registerFunction('onSignIn', fnc_list, ['googleUser'])
        return g_button

    def facebook_sign_in(self, client_id, scopes=['public_profile', 'email'], insert_button=True, graphVersion='v7.0', type='login_with', debug=False):
        """

        :param client_id:
        :param insert_button:
        :param graphVersion:
        :param type:
        :param debug:
        :return:
        """
        accepted_types = ('login_with', 'continue_with')

        if type not in accepted_types:
            raise Exception('Specified type: %s not in accepted values. Accepted values are as follows: %s' % (type, ', '.join(accepted_types)))

        getFbSdk = '''(function(d, s, id) {
                            var js, fjs = d.getElementsByTagName(s)[0];
                            if (d.getElementById(id)) return;
                            js = d.createElement(s); js.id = id;
                            js.src = "https://connect.facebook.net/en_US/sdk.js";
                            fjs.parentNode.insertBefore(js, fjs);
                          }(document, 'script', 'facebook-jssdk'));
                            '''

        returnFbResponse = '''function statusChangeCallback(response) {
                                return response;
                                }
        '''
        jsCheckLogin = '''function checkLoginState() {               
                            FB.getLoginStatus(function(response) {   
                              return response;
                            });
                          }'''
        jsInitFnc = '''window.fbAsyncInit = function() {FB.init({
                        appId: '%s',
                        cookie: true,
                        xfbml: true,
                        version: '%s'
                        });
                    };''' % (client_id, graphVersion)

        self._report.js.onReady([getFbSdk, returnFbResponse, jsCheckLogin, jsInitFnc])
        fb_button = self._report.ui.div()
        fb_button.style.no_class()
        fb_button.style.clear_style()
        fb_button.style.add_classes.external('fb-login-button')
        fb_button.attr.update({'data-size': 'large', 'data-button-type': type, 'data-layout': 'default', 'data-auto-logout-link': 'true',
                               'data-use-continue-as': 'false', 'data-width': '', 'onlogin': "checkLoginState();", 'scopes': ','.join(scopes)})



import random


def getSeries(count, size, negatives=0.1, missing=0.2):
  data = []
  #
  neg = size * [False]
  miss = size * [False]
  for s in range(size):
    data.append({"x": s, 'r': random.randint(0, 10), 'g': random.randint(0, 5)})
    for c in range(count):
      if miss[s]:
        continue

      data[-1][c] = random.randint(0, 10000) / 100 * (-1 if neg[s] else 1)
  return data


languages = [
  {"name": 'C', 'type': 'code', 'rating': 17.07, 'change': 12.82},
  {"name": 'Java', 'type': 'code', 'rating': 16.28, 'change': 0.28},
  {"name": 'Python', 'type': 'script', 'rating': 9.12, 'change': 1.29},
  {"name": 'C++', 'type': 'code', 'rating': 6.13, 'change': -1.97},
  {"name": 'C#', 'type': 'code', 'rating': 4.29, 'change': 0.3},
  {"name": 'Visual Basic', 'type': 'script', 'rating': 4.18, 'change': -1.01},
  {"name": 'JavaScript', 'type': 'script', 'rating': 2.68, 'change': -0.01},
  {"name": 'PHP', 'type': 'script', 'rating': 2.49, 'change': 0},
  {"name": 'SQL', 'type': 'script', 'rating': 2.09, 'change': -0.47},
  {"name": 'R', 'type': 'script', 'rating': 1.85, 'change': 0.90},
]


import os
import time
import json

from epyk.core.js import Imports
from epyk.core.js import Js
from epyk.core.js import JsUtils

from epyk.core.html.templates import HtmlTmplBase


class OutBrowsers(object):
  def __init__(self, context):
    self._context = context

  def codepen(self, path=None, target="_blank", open_browser=True):
    """
    Description:
    ------------
    Update the Html launcher and send the data to codepen
    URL used: https://codepen.io/pen/define/

    Related Pages:

      https://www.debuggex.com/cheatsheet/regex/python

    Attributes:
    ----------
    :param path: String. Output path in which the static files will be generated
    :param target: String. Load the data in a new tab in the browser
    :param open_browser: Boolean. Flag to open the browser automatically

    :return: The output launcher full file name
    """
    import re
    import webbrowser

    results = self._context._to_html_obj()
    js_external = re.findall('<script language="javascript" type="text/javascript" src="(.*?)"></script>', results['jsImports'])
    css_external = re.findall('<link rel="stylesheet" href="(.*?)" type="text/css">', results['cssImports'])
    jsObj = Js.JsBase()
    result = {"js": results["jsFrgs"], "js_external": ";".join(js_external), "css_external": ";".join(css_external), "html": results['content'], "css": results["cssStyle"]}
    data = jsObj.location.postTo("https://codepen.io/pen/define/", {"data": json.dumps(result)}, target=target)
    if path is None:
      path = os.path.join(os.getcwd(), "outs")
    else:
      path = os.path.join(path)
    if not os.path.exists(path):
      os.makedirs(path, exist_ok=True)
    with open(os.path.join(path, "RunnerCodepen.html"), "w") as f:
      f.write('<html><body></body><script>%s</script></html>' % data.replace("\\\\n", ""))
    launcher_file = os.path.join(path, "RunnerCodepen.html")
    if open_browser:
      webbrowser.open(launcher_file)
    return launcher_file

  def stackblitz(self, path=None, target="_blank", open_browser=True):
    """
    Description:
    ------------

    Related Pages:

      https://stackblitz.com/docs

    Attributes:
    ----------
    :param path: String. Output path in which the static files will be generated
    :param target: String. Load the data in a new tab in the browser
    :param open_browser: Boolean. Flag to open the browser automatically
    """
    import webbrowser

    results = self._context._to_html_obj()
    results['jsFrgs'] = results['jsFrgs'].replace('"', "'")
    results['cssImports'] = results['cssImports'].replace('"', "'")
    results['content'] = results['content'].replace('"', "'")
    with open(os.path.join(path, "RunnerStackblitz.html"), "w") as f:
      f.write('''
<html lang="en">
<head></head>
<body>
<form id="mainForm" method="post" action="https://stackblitz.com/run" target="_self">
<input type="hidden" name="project[files][index.js]" value="
%(jsFrgs)s
">
<input type="hidden" name="project[files][index.html]" value="
%(cssImports)s
<style>
%(cssStyle)s
</style>
%(content)s
">
<input type="hidden" name="project[description]" value="Epyk Example">
<input type="hidden" name="project[dependencies]" value="{&quot;rxjs&quot;:&quot;5.5.6&quot;}">
<input type="hidden" name="project[template]" value="javascript">
<input type="hidden" name="project[settings]" value="{&quot;compile&quot;:{&quot;clearConsole&quot;:false}}">
</form>
<script>document.getElementById("mainForm").submit();</script>

</body></html>
''' % results)
    launcher_file = os.path.join(path, "RunnerStackblitz.html")
    if open_browser:
      webbrowser.open(launcher_file)
    return launcher_file


class PyOuts(object):
  def __init__(self, report=None, options=None):
    self._report, self._options = report, options
    self.excluded_packages, html_tmpl = None, HtmlTmplBase.JUPYTERLAB

  def _to_html_obj(self, htmlParts=None, cssParts=None, split_js=False):
    """
    Description:
    ------------
    Create the HTML result object from the report definition

    Attributes:
    ----------
    :param htmlParts: Optional. HTML Content of the page
    :param cssParts: Optional. CSS classes content of the page

    :return: A python dictionary with the HTML results
    """
    order_components = list(self._report.components.keys())
    if htmlParts is None:
      htmlParts, cssParts = [], {}
      for component_id in order_components:
        component = self._report.components[component_id]
        if component.name == 'Body':
          continue

        if component.options.managed:
          htmlParts.append(component.html())
        #
        cssParts.update(component.style.get_classes_css())
    onloadParts, onloadPartsCommon = list(self._report._jsText), {}
    for data_id, data in self._report._props.get("data", {}).get('sources', {}).items():
      onloadParts.append("var data_%s = %s" % (data_id, json.dumps(data)))

    for k, v in self._report._props.get('js', {}).get('functions', {}).items():
      sPmt = "(%s)" % ", ".join(list(v["pmt"])) if "pmt" in v else "{}"
      onloadParts.append("function %s%s{%s}" % (k, sPmt, v["content"].strip()))

    if split_js:
      onloadPartsCommon = self._report._props.get('js', {}).get("constructors", {})
    else:
      for c, d in self._report._props.get('js', {}).get("constructors", {}).items():
        onloadParts.append(d)
    for c, d in self._report._props.get('js', {}).get("configs", {}).items():
      onloadParts.append(str(d))

    for c, d in self._report._props.get('js', {}).get("datasets", {}).items():
      onloadParts.append(d)

    for b in self._report._props.get('js', {}).get("builders", []):
      onloadParts.append(b)

    for b in self._report._props.get('js', {}).get("builders_css", []):
      onloadParts.append(b)

    # Add the component on ready functions
    for component_id in order_components:
      component = self._report.components[component_id]
      if component.name == 'Body':
        continue

      onloadParts.extend(component._browser_data['component_ready'])

      for event, source_fncs in component._browser_data['mouse'].items():
        for source, event_fncs in source_fncs.items():
          str_fncs = JsUtils.jsConvertFncs(event_fncs['content'], toStr=True)
          onloadParts.append("%s.addEventListener('%s', function(event){%s})" % (source, event, str_fncs))

      for event, source_fncs in component._browser_data['keys'].items():
        for source, event_fncs in source_fncs.get_event().items():
          str_fncs = JsUtils.jsConvertFncs(event_fncs['content'], toStr=True)
          onloadParts.append("%s.addEventListener('%s', function(event){%s})" % (source, event, str_fncs))

    # Add the page on document ready functions
    for on_ready_frg in self._report._props.get('js', {}).get('onReady', []):
      onloadParts.append(on_ready_frg)
    # Add the document events functions
    for event, source_fncs in self._report._props.get('js', {}).get('events', []).items():
      for source, event_fncs in source_fncs.get_event().items():
        str_fncs = JsUtils.jsConvertFncs(event_fncs['content'], toStr=True)
        onloadParts.append("%s.addEventListener('%s', function(event){%s})" % (source, event, str_fncs))

    importMng = Imports.ImportManager(online=True, report=self._report)
    results = {
      'cssStyle': "%s\n%s" % ("\n".join([v for v in cssParts.values()]), "\n".join(self._report._cssText)),
      'cssContainer': ";".join(["%s:%s" % (k, v) for k, v in self._report._props.get('css', {}).get('container', {}).items()]),
      'content': "\n".join(htmlParts),
      'jsFrgsCommon': onloadPartsCommon, # This is only used in some specific web frameworks and it is better to keep the data as list
      'jsFrgs': ";".join(onloadParts),
      'cssImports': importMng.cssResolve(self._report.cssImport, self._report.cssLocalImports, excluded=self.excluded_packages),
      'jsImports': importMng.jsResolve(self._report.jsImports, self._report.jsLocalImports, excluded=self.excluded_packages)
    }
    return results

  def _repr_html_(self):
    """
    Description:
    ------------
    Standard output for Jupyter Notebooks.

    This is what will use IPython in order to display the results in cells.
    """
    results = self._to_html_obj()
    importMng = Imports.ImportManager(online=True, report=self._report)
    require_js = importMng.to_requireJs(results, self.excluded_packages)
    results['paths'] = "{%s}" % ", ".join(["%s: '%s'" % (k, p) for k, p in require_js['paths'].items()])
    results['jsFrgs_in_req'] = require_js['jsFrgs']
    return self.html_tmpl.strip() % results

  def jupyterlab(self):
    """
    Description:
    ------------
    For a display of the report in JupyterLab.
    Thanks to this function some packages will not be imported to not conflict with the existing ones

    Related Pages:

      https://jupyter.org/
    """
    self.html_tmpl = HtmlTmplBase.JUPYTERLAB
    self.excluded_packages = ['bootstrap']
    return self

  def jupyter(self):
    """
    Description:
    ------------
    For a display of the report in Jupyter.
    Thanks to this function some packages will not be imported to not conflict with the existing ones

    Related Pages:

      https://jupyter.org/

    :return: The ouput object with the function _repr_html_
    """
    self.html_tmpl = HtmlTmplBase.JUPYTER
    try:
      import notebook

      self.excluded_packages = []
      nb_path = os.path.split(notebook.__file__)[0]
      for f in os.listdir(os.path.join(nb_path, 'static', 'components')):
        self.excluded_packages.append(Imports.NOTEBOOK_MAPPING.get(f, f))
    except:
      self.excluded_packages = ['bootstrap', 'jquery', 'moment', 'jqueryui', 'mathjs']
    return self

  def w3cTryIt(self, path=None, name=None):
    """
    Description:
    ------------
    This will produce everything in a single page which can be directly copied to the try editor in w3C website

    Related Pages:

      https://www.w3schools.com/html/tryit.asp?filename=tryhtml_basic

    Attributes:
    ----------
    :param path: The path in which the output files will be created
    :param name: The filename without the extension

    """
    if path is None:
      path = os.path.join(os.getcwd(), "outs", "w3schools")
    else:
      path = os.path.join(path, "w3schools")
    if not os.path.exists(path):
      os.makedirs(path, exist_ok=True)
    if name is None:
      name = int(time.time())
    file_path = os.path.join(path, "%s.html" % name)
    with open(file_path, "w") as f:
      f.write(self._repr_html_())
    return file_path

  def codepen(self, path=None, name=None):
    """
    Description:
    ------------

    Usage::

      Related Pages:

      https://codepen.io/

    Attributes:
    ----------
    :param path: The path in which the output files will be created
    :param name: The filename without the extension

    TODO Try to add the prefill
    https://blog.codepen.io/documentation/api/prefill/

    :return: The file path
    """
    self.jsfiddle(path, name, framework="codepen")

  def jsfiddle(self, path=None, name=None, framework="jsfiddle"):
    """
    Description:
    ------------
    Produce files which can be copied directly to https://jsfiddle.net in order to test the results and perform changes.

    The output is always in a sub directory jsfiddle

    Usage::

      Related Pages:

      https://jsfiddle.net/

    Attributes:
    ----------
    :param path: The path in which the output files will be created
    :param name: The filename without the extension

    :return: The file path
    """
    if path is None:
      path = os.path.join(os.getcwd(), "outs", framework)
    else:
      path = os.path.join(path, framework)
    if not os.path.exists(path):
      os.makedirs(path, exist_ok=True)
    if os.path.exists(path):
      if name is None:
        name = int(time.time())
      results = self._to_html_obj()
      # For the JavaScript builders
      with open(os.path.join(path, "%s.js" % name), "w") as f:
        f.write(results["jsFrgs"])

      # For all the doms and imports
      with open(os.path.join(path, "%s.html" % name), "w") as f:
        f.write("%s\n" % results["cssImports"])
        f.write("%s\n" % results["jsImports"])
        f.write(results["content"])

      # For the CSS styles
      with open(os.path.join(path, "%s.css" % name), "w") as f:
        f.write(results["cssStyle"])
    return path

  def html_file(self, path=None, name=None, split_files=False):
    """
    Description:
    ------------
    Function used to generate a static HTML page for the report

    Usage::

      Attributes:
    ----------
    :param path: The path in which the output files will be created
    :param name: The filename without the extension

    :return: The file full path
    """
    if path is None:
      path = os.path.join(os.getcwd(), "outs")
    if not os.path.exists(path):
      os.makedirs(path, exist_ok=True)
    if name is None:
      name = int(time.time())
    html_file_path = os.path.join(path, "%s.html" % name)
    htmlParts = []
    cssParts = dict(self._report.body.style.get_classes_css())
    order_components = list(self._report.components.keys())
    for component_id in order_components:
      component = self._report.components[component_id]
      if component.name == 'Body':
        continue

      if component.options.managed:
        htmlParts.append(component.html())
      cssParts.update(component.style.get_classes_css())
    body = str(self._report.body.set_content(self._report, "\n".join(htmlParts)))
    results = self._to_html_obj(htmlParts, cssParts, split_js=split_files)
    if split_files:
      results['cssImports'] = '%s\n<link rel="stylesheet" href="./css/%s.css" type="text/css">\n\n' % (results['cssImports'], name)
      body = '%s\n\n<script language="javascript" type="text/javascript" src="./js/%s.js"></script>' % (body, name)
      if not os.path.exists(os.path.join(path, 'css')):
        os.makedirs(os.path.join(path, 'css'), exist_ok=True)
      with open(os.path.join(path, 'css', "%s.css" % name), "w") as f:
        f.write(results['cssStyle'])

      if not os.path.exists(os.path.join(path, 'js')):
        os.makedirs(os.path.join(path, 'js'), exist_ok=True)
      with open(os.path.join(path, 'js', "%s.js" % name), "w") as f:
        f.write(";".join(results['jsFrgsCommon'].values()))

    # Add the worker sections when no server available
    for js_id, wk_content in self._report._props.get('js', {}).get("workers", {}).items():
      body += '\n<script id="%s" type="javascript/worker">\n%s\n</script>' % (js_id, wk_content)
    with open(html_file_path, "w") as f:
      results['body'] = body
      results['header'] = self._report.headers
      f.write(HtmlTmplBase.STATIC_PAGE % results)
    return html_file_path

  def web(self):
    """
    Description:
    ------------
    Return the complete page structure to allow the various web framework to split the code accordingly.
    Fragments will then be used by the various framework to create the corresponding pages
    """
    htmlParts = []
    cssParts = dict(self._report.body.style.get_classes_css())
    order_components = list(self._report.components.keys())
    for component_id in order_components:
      component = self._report.components[component_id]
      if component.name == 'Body':
        continue

      if component.options.managed:
        htmlParts.append(component.html())
      cssParts.update(component.style.get_classes_css())
    body = str(self._report.body.set_content(self._report, "\n".join(htmlParts)))
    results = self._to_html_obj(htmlParts, cssParts, split_js=True)
    results['body'] = body.replace("<body", "<div").replace("</body>", "</div>")
    return results

  def publish(self, server, app_path, selector, name=None, module=None, target_folder="apps", auto_route=False):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param server:
    :param app_path:
    :param name:
    """
    from epyk.web import angular, node, vue, react, deno

    app = None
    if server.upper() == 'NODE':
      app = node.Node(app_path=app_path, name=name or 'node')
      app.page(report=self._report, selector=selector or 'app-root', name=module)
    elif server.upper() == 'DENO':
      app = deno.Deno(app_path=app_path, name=name or 'deno')
      app.page(report=self._report, selector=selector or 'app-root', name=module)
    elif server.upper() == 'ANGULAR':
      app = angular.Angular(app_path=app_path, name=name or 'angular')
      component = module or selector.capitalize()
      app.page(report=self._report, selector=selector, name=component, target_folder=target_folder)
      if auto_route:
        app.route().add(component, selector, "apps")
    elif server.upper() == 'VUE':
      app = vue.VueJs(app_path=app_path, name=name or 'vue')
      app.page(report=self._report, selector=selector or 'app-root', name=module)
    elif server.upper() == 'REACT':
      app = react.React(app_path=app_path, name=name or 'react')
      app.page(report=self._report, selector=selector or 'app-root', name=module)
    app.publish()

  def markdown_file(self, path=None, name=None):
    """
    Description:
    ------------
    Writes a Markdown file from the report object

    Attributes:
    ----------
    :param path: The path in which the output files will be created
    :param name: The filename without the extension

    :return: The file path
    """
    if path is None:
      path = os.path.join(os.getcwd(), "outs", "markdowns")
    else:
      path = os.path.join(path, "markdowns")
    if not os.path.exists(path):
      os.makedirs(path, exist_ok=True)
    if os.path.exists(path):
      if name is None:
        name = "md_%s.amd" % int(time.time())

      file_Path = os.path.join(path, name)
      with open(file_Path, "w") as f:
        order_components = list(self._report.components.keys())
        for component_id in order_components:
          component = self._report.components[component_id]
          if component.name == 'Body':
            continue

          if hasattr(component, "to_markdown"):
            f.write("%s\n" % component.to_markdown(component.vals))
      return file_Path

  def html(self):
    """

    :return:
    """
    self.html_tmpl = HtmlTmplBase.STATIC_PAGE
    results = self._to_html_obj()
    importMng = Imports.ImportManager(online=True, report=self._report)
    require_js = importMng.to_requireJs(results, self.excluded_packages)
    results['paths'] = "{%s}" % ", ".join(["%s: '%s'" % (k, p) for k, p in require_js['paths'].items()])
    results['jsFrgs_in_req'] = require_js['jsFrgs']
    htmlParts = []
    cssParts = dict(self._report.body.style.get_classes_css())
    order_components = list(self._report.components.keys())
    for component_id in order_components:
      component = self._report.components[component_id]
      if component.name == 'Body':
        continue

      if component.options.managed:
        htmlParts.append(component.html())
      cssParts.update(component.style.get_classes_css())
    body = str(self._report.body.set_content(self._report, "\n".join(htmlParts)))
    results['body'] = body
    results['header'] = self._report.headers
    return self.html_tmpl.strip() % results

  @property
  def browser(self):
    """
    Description:
    ------------
    This module will require the package webbrowser.
    It will allow outputs to be created directly in the webpages (without using intermediary text files
    """
    return OutBrowsers(self)




import os

import subprocess

from epyk.core import Page
from epyk.web import node


JS_MODULES_IMPORTS = {
  'showdown': '''
import Showdown from 'showdown';
var showdown = Showdown;
'''
}


class NpxCli(object):
  def __init__(self, vue_app_path):
    self._vue_app_path = vue_app_path

  def create(self, name=None):
    """
    Description:
    ------------
    The create-react-app is an officially supported way to create React applications.

    Related Pages:

      https://fr.reactjs.org/docs/create-a-new-react-app.html
    https://www.w3schools.com/react/default.asp

    Attributes:
    ----------
    :param name:
    """
    if name is None:
      subprocess.run('npx create-react-app --help', shell=True, cwd=self._vue_app_path)
    else:
      subprocess.run('npx create-react-app %s' % name, shell=True, cwd=self._vue_app_path)

  def build(self):
    """
    Description:
    ------------
    Builds the app for production to the build folder.
    It correctly bundles React in production mode and optimizes the build for the best performance.

    Related Pages:

      https://create-react-app.dev/docs/getting-started/
    """
    subprocess.run('npm run build', shell=True, cwd=self._vue_app_path)

  def start(self):
    """
    Description:
    ------------
    Runs the app in development mode. Open http://localhost:3000 to view it in the browser.

    Related Pages:

      https://create-react-app.dev/docs/getting-started/
    """
    subprocess.run('npm test', shell=True, cwd=self._vue_app_path)

  def test(self):
    """
    Description:
    ------------
    Runs the test watcher in an interactive mode. By default, runs tests related to files changed since the last commit.

    Related Pages:

      https://create-react-app.dev/docs/getting-started/
    """
    subprocess.run('npm test', shell=True, cwd=self._vue_app_path)


class App(object):

  def __init__(self, app_path, app_name, alias, name, report=None):
    self.imports = {}
    self.vars, self.__map_var_names, self._report = {}, {}, report
    self._app_path, self._app_name = app_path, app_name
    self.alias, self.__path, self.className, self.__components = alias, 'apps', name, None
    self.comps = {}

  @property
  def name(self):
    """
    Description:
    ------------
    Return the prefix of the component module (without any extension)
    """
    return self.className

  @property
  def path(self):
    """
    Description:
    ------------
    Return the full path of the component modules
    """
    return os.path.join("./", self.__path, self.name).replace("\\", "/")

  def export(self, path=None, target_path=None):
    """
    Description:
    ------------

    Attributes:
    ----------
    :param path:
    :param target_path: for example ['src', 'app']
    """
    self.__path = path or self.__path
    if target_path is None:
      target_path = []
    target_path.append(self.__path)
    module_path = os.path.join(self._app_path, self._app_name, *target_path)
    if not os.path.exists(module_path):
      os.makedirs(module_path)

    page = self._report.outs.web()
    page['alias'] = self.alias
    page['name'] = self.name
    page['js_module'] = self.alias.replace("-", "_")
    page['builders'] = ", ".join(list(page['jsFrgsCommon'].keys()))
    with open(os.path.join(module_path, "%s.css" % self.name), "w") as f:
      f.write(page['cssStyle'])
    # export default %(alias)s
    with open(os.path.join(module_path, "%s.js" % self.name), "w") as f:
      f.write('''
import React from 'react';
import ReactDOM from 'react-dom';
import { %(builders)s } from './%(js_module)s.js'
import './%(name)s.css';

class %(alias)s extends React.Component {
  render() {
    return %(body)s;
  }
}

export default AppRoot;
ReactDOM.render(<%(alias)s />, document.getElementById('root'); %(jsFrgs)s);
''' % page)

    with open(os.path.join(module_path, "%s.js" % page['js_module']), "w") as f:
      for js_dep in JS_MODULES_IMPORTS:
        if js_dep in self._report.jsImports:
          f.write("%s\n" % JS_MODULES_IMPORTS[js_dep])
      for buider in page['jsFrgsCommon'].values():
        f.write("export %s;\n" % buider)


class React(node.Node):

  def cli(self, app_name=None):
    """
    Description:
    ------------
    Create React App is an officially supported way to create single-page React applications.
    It offers a modern build setup with no configuration.

    Related Pages:

      https://create-react-app.dev/docs/getting-started

    Attributes:
    ----------
    :param app_name: String. The React.js application name
    """
    return NpxCli(self._app_path)

  def page(self, selector=None, name=None, report=None, auto_route=False):
    """
    Description:
    ------------
    Create a specific Application as a component in the Angular framework.

    Unlike a basic component, the application will be routed to be accessed directly.

    Description:
    ------------
    :param report: Object. A report object
    :param selector: String. The url route for this report in the Angular app
    :param name: String. The component classname in the Angular framework
    """
    if name is None:
      script = os.path.split(sys._getframe().f_back.f_code.co_filename)[1][:-3]
      name = "".join([s.capitalize() for s in script.split("_")])
      if selector is None:
        selector = script.replace("_", "-")
    report = report or Page.Report()
    report.framework("VUE")
    self._page = App(self._app_path, self._app_name, selector, name, report=report)
    #if auto_route:
    #  self.route.add(self.__page.className, self.__page.alias, self.__page.path)
    return self._page

  def publish(self, app_name=None, target_path=None):
    """
    Description:
    ------------
    Publishh the Vue.js application

    Attributes:
    ----------
    :param app_name:
    :param target_path: List  for example ['src', 'app']
    """
    if self._page is not None:
      self._page.export(target_path=target_path)
    #if self.__route is not None:
    #  self.route.export()



from epyk.core.js import JsUtils
from epyk.core.js.primitives import JsObjects


class SocketState(object):
  CONNECTING = 0
  OPEN = 1
  CLOSING = 2
  CLOSED = 2


class HttpCode(object):
  NORMAL_CLOSURE = 1000
  SHUTDOWN_SERVER = 1001
  LOST_CONNECTION = 1006

  MESSAGE_TYPE_NOT_CONSISTENT = 1007
  MESSAGE_VIOLATE_POLICY = 1008
  MESSAGE_TOO_BIG = 1009

  UNEXPECTED_ERROR = 1011


class WebSocket(object):

  def __init__(self, htmlCode=None, src=None, secured=False):
    """
    Description:
    ------------
    """
    self._src, self.secured, self.__connect = src, secured, None
    self._selector = htmlCode or "websocket_%s" % id(self)

  @property
  def readyState(self):
    """

    """
    return JsObjects.JsObject.JsObject.get("%s.readyState" % self._selector)

  @property
  def states(self):
    """
    To get connection state, additionally there’s socket.readyState property with values:

    Related Pages:

      https://javascript.info/websocket
    """
    return SocketState

  @property
  def http_codes(self):
    """
    To get connection state, additionally there’s socket.readyState property with values:

    Related Pages:

      https://tools.ietf.org/html/rfc6455#section-7.4.1
    """
    return HttpCode

  @property
  def message(self):
    """

    """
    return JsObjects.JsObject.JsObject.get("event.data")

  def reconnect(self):
    return JsObjects.JsVoid("if(%(varName)s.readyState > 1){ %(varName)s = %(connect)s}" % {"varName": self._selector, "connect": self.__connect})

  def connect(self, url=None, port=None, protocol=None, from_config=None):
    """
    Description:
    ------------
    n order to communicate using the WebSocket protocol, you need to create a WebSocket object; this will automatically attempt to open the connection to the server.

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param url: String. The URL to which to connect; this should be the URL to which the WebSocket server will respond.
        This should use the URL scheme wss://, although some software may allow you to use the insecure ws:// for local connections.
    :param protocol: String or List. Either a single protocol string or an array of protocol strings.
    :param from_config:
    """
    if from_config is not None:
      self._src._props['js']['builders'].add("var %s = new WebSocket(%s)" % (self._selector, from_config.address))
      self.__connect = "new WebSocket(%s)" % from_config.address
      return JsObjects.JsVoid("var %s = new WebSocket(%s)" % (self._selector, from_config.address))

    prefix = "wss" if self.secured else 'ws'
    if protocol is None:
      self._src._props['js']['builders'].add(
        "var %s = new WebSocket('%s://%s:%s')" % (self._selector, prefix, url, port))
      self.__connect = "new WebSocket('%s://%s:%s')" % (prefix, url, port)
      return JsObjects.JsVoid("var %s = new WebSocket('%s://%s:%s')" % (self._selector, prefix, url, port))

    protocol = JsUtils.jsConvertData(protocol, None)
    self._src._props['js']['builders'].add(
      "var %s = new WebSocket('%s://%s:%s', %s)" % (self._selector, prefix, url, port, protocol))
    self.__connect = "new WebSocket('%s://%s:%s', %s)" % (prefix, url, port, protocol)
    return JsObjects.JsVoid("var %s = new WebSocket('%s://%s:%s', %s)" % (self._selector, prefix, url, port, protocol))

  def onopen(self, jsFncs):
    """
    Description:
    ------------

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onopen = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def onmessage(self, jsFncs):
    """
    Description:
    ------------

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onmessage = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def onerror(self, jsFncs):
    """
    Description:
    ------------

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onerror = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def onclose(self, jsFncs):
    """
    Description:
    ------------

    Related Pages:

      https://javascript.info/websocket

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onclose = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def receive(self, jsFncs):
    """
    Description:
    ------------

    :param jsFncs:
    """
    return JsObjects.JsVoid("%(varName)s.onmessage = function (event) { %(data)s }" % {"varName": self._selector, "data": JsUtils.jsConvertFncs(jsFncs, toStr=True)})

  def send(self, data):
    """
    Description:
    ------------
    Basic way to send a text message to the server

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param data: String. The message to be sent
    """
    data = JsUtils.jsConvertData(data, None)
    return JsObjects.JsVoid("%(varName)s.send(%(data)s)" % {"varName": self._selector, "connect": self.__connect, "data": data})

  def sendText(self, components, attrs=None):
    """
    Description:
    ------------
    Send a complex message from components.

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param components: List. The list of HTML components (it will get the dom.content automatically)
    :param attrs: Dictionary. Attach some static attributes to the request
    """
    from epyk.core.data import primitives
    from epyk.core.data import datamap

    dftl_attrs = {"type": 'message', 'date': primitives.date()}
    if attrs is not None:
      dftl_attrs.update(attrs)
    data = JsUtils.jsConvertData(datamap(components, attrs=dftl_attrs), None)
    return JsObjects.JsVoid("%(varName)s.send(JSON.stringify(%(data)s))" % {"varName": self._selector, "connect": self.__connect, "data": data})

  def close(self, code=1000, reason=None):
    """
    Description:
    ------------
    When you've finished using the WebSocket connection, call the WebSocket method close()

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param code: Integer. The HTTP code to be sent to the server for the closure
    :param reason: String. The message to be sent to the server for the closure
    """
    if reason is None:
      return JsObjects.JsVoid("%s.close(%s)" % (self._selector, code))

    return JsObjects.JsVoid("%s.close(%s, '%s')" % (self._selector, code, reason))


class Worker(object):

  def __init__(self, htmlCode=None, src=None, server=False):
    """
    Description:
    ------------

    """
    self._src, self.__server = src, server
    self._selector = htmlCode or "worker_%s" % id(self)
    self._src._props['js']['builders'].add("var %s" % self._selector)

  @property
  def message(self):
    """
    Description:
    ------------

    """
    return JsObjects.JsObject.JsObject.get("event.data")

  def connect(self, script=None, content=None):
    """
    Description:
    ------------

    Related Pages:

      https://www.w3schools.com/html/html5_webworkers.asp
      https://www.html5rocks.com/en/tutorials/workers/basics/

    Attributes:
    ----------
    :param script: String.
    :param content: String.
    """
    if not self.__server or content is not None:
      # load the file and put this to Js Blobs
      if script is not None:
        with open(script) as f:
          self._src._props['js']['workers']['js_%s' % self._selector] = f.read()
      else:
        self._src._props['js']['workers']['js_%s' % self._selector] = content
      self._src._props['js']['builders'].add('''
        var blob_%(selector)s = new Blob([document.querySelector('#js_%(selector)s').textContent ], {type: "text/javascript"})
        %(selector)s = new Worker(window.URL.createObjectURL(blob_%(selector)s))''' % {'selector': self._selector})
    else:
      self._src._props['js']['builders'].add("%s = new Worker('%s')" % (self._selector, script))
    return JsObjects.JsVoid("%s = new Worker('%s')" % (self._selector, script))

  def postMessage(self, data):
    """
    Description:
    ------------

    Related Pages:

      https://www.html5rocks.com/en/tutorials/workers/basics/

    Attributes:
    ----------
    :param data:
    """
    data = JsUtils.jsConvertData(data, None)
    return JsObjects.JsVoid("%s.postMessage(%s)" % (self._selector, data))

  def on(self, eventType, jsFncs):
    self._src.js.onReady(self.addEventListener(eventType, jsFncs))

  def addEventListener(self, eventType, jsFncs):
    """
    Description:
    ------------

    :param eventType:
    :param jsFncs:
    """
    return JsObjects.JsVoid("%(varName)s.addEventListener('%(eventType)s', function (event) {%(data)s})" % {"varName": self._selector, 'eventType': eventType, "data": JsUtils.jsConvertFncs(jsFncs, toStr=True)})

  def receive(self, jsFncs):
    """
    Description:
    ------------

    :param jsFncs:
    """
    return JsObjects.JsVoid("%(varName)s.onmessage = function (event) {%(data)s}" % {"varName": self._selector, "data": JsUtils.jsConvertFncs(jsFncs, toStr=True)})

  def terminate(self):
    """
    Description:
    ------------
    When a web worker object is created, it will continue to listen for messages (even after the external script is finished) until it is terminated.

    Related Pages:

      https://www.w3schools.com/html/html5_webworkers.asp
    """
    return JsObjects.JsVoid("%s.terminate(); %s = undefined" % (self._selector, self._selector))

  def close(self):
    """
    Description:
    ------------
    Proxy to the terminate method
    """
    return self.terminate()


class ServerSentEvent(object):

  def __init__(self, htmlCode=None, src=None, server=False):
    """
    Description:
    ------------

    """
    self._src, self.__server = src, server
    self._selector = htmlCode or "sse_%s" % id(self)
    self._src._props['js']['builders'].add("var %s" % self._selector)

  @property
  def message(self):
    """

    """
    return JsObjects.JsObject.JsObject.get("event.data")

  def connect(self, url=None, port=None, from_config=None, options=None):
    """
    Description:
    ------------
    n order to communicate using the WebSocket protocol, you need to create a WebSocket object; this will automatically attempt to open the connection to the server.

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/WebSockets_API/Writing_WebSocket_client_applications

    Attributes:
    ----------
    :param url: String. The URL to which to connect; this should be the URL to which the WebSocket server will respond.
        This should use the URL scheme wss://, although some software may allow you to use the insecure ws:// for local connections.
    :param protocol: String or List. Either a single protocol string or an array of protocol strings.
    :param from_config:
    """
    if from_config is not None:
      self._src._props['js']['builders'].add("%s = new EventSource(%s)" % (self._selector, from_config.address))
      self.__connect = "new EventSource(%s)" % from_config.address
      return JsObjects.JsVoid("%s = new EventSource(%s)" % (self._selector, from_config.address))

    self._src._props['js']['builders'].add("%s = new EventSource('%s:%s')" % (self._selector, url, port))
    self.__connect = "new EventSource('%s:%s')" % (url, port)
    return JsObjects.JsVoid("%s = new EventSource('%s:%s')" % (self._selector, url, port))

  def onmessage(self, jsFncs):
    """
    Description:
    ------------
    The EventSource object is used to receive server-sent event notifications:

    Related Pages:

      https://www.w3schools.com/html/html5_serversentevents.asp

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onmessage = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def onerror(self, jsFncs):
    """
    Description:
    ------------
    When an error occurs

    Related Pages:

      https://www.w3schools.com/html/html5_serversentevents.asp

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onerror = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def onopen(self, jsFncs):
    """
    Description:
    ------------
    When a connection to the server is opened

    Related Pages:

      https://www.w3schools.com/html/html5_serversentevents.asp

    Attributes:
    ----------
    :param jsFncs:
    """
    if not isinstance(jsFncs, list):
      jsFncs = [jsFncs]
    self._src.js.onReady("%s.onopen = function (event) { %s }" % (self._selector, JsUtils.jsConvertFncs(jsFncs, toStr=True)))
    return self

  def addEventListener(self, eventType, jsFncs):
    """
    Description:
    ------------

    :param eventType:
    :param jsFncs:
    """
    return JsObjects.JsVoid("%(varName)s.addEventListener('%(eventType)s', function (event) {%(data)s})" % {"varName": self._selector, 'eventType': eventType, "data": JsUtils.jsConvertFncs(jsFncs, toStr=True)})

  def on(self, eventType, jsFncs):
    self._src.js.onReady(self.addEventListener(eventType, jsFncs))

  def receive(self, jsFncs):
    """
    Description:
    ------------
    The EventSource object is used to receive server-sent event notifications:

    Related Pages:

      https://www.w3schools.com/html/html5_serversentevents.asp

    Attributes:
    ----------
    :param jsFncs:
    """
    return JsObjects.JsVoid("%(varName)s.onmessage = function (event) { %(data)s }" % {"varName": self._selector, "data": JsUtils.jsConvertFncs(jsFncs, toStr=True)})

  def close(self):
    """
    Description:
    ------------
    By default, if the connection between the client and server closes, the connection is restarted.
    The connection is terminated with the .close() method.

    Related Pages:

      https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events
    """
    return JsObjects.JsVoid("%s.close(); %s = undefined" % (self._selector, self._selector))



from epyk.core.Page import Report
rptObj = Report()




rptObj.auth.google_sign_in('739797595806-8kb4iili6br2gt00h5uhuugkvjhlq4ho.apps.googleusercontent.com', insert_button=True, debug=True)
print(rptObj.outs.html_file(path=r"C:\Users\nelso\Desktop", name='g_signin'))



"""

epyk_npm.exe
"""


def install():
  """
  Install the package following the below route:
    - Test if the package existing in NPM
    - Test if the package is availalbe on CDNJS
  """


def add():
  """
  Add an extra external package in the project repository
  """


def info():
  """
  Get package information from NPM repository and check if the current version in the project is the same
  """


def cdnjs():
  """
  Get package information from CDNJS
  """



"""

epyk_export.exe
"""


def angular():
  """
  Generate an Angular Application from the Epyk page
  """


def vue():
  pass


def react():
  pass


def node():
  pass


def jupyter():
  pass


def deno():
  pass


def html():
  pass


def serve():
  pass



"""
Main command lines for Epyk

epyk.exe
"""


def new():
  """
  Create a new Epyk Structure.

  THis structure should be easy to integrate any Python exnsting project:

  The structure can be the ones below
  ui/
    pages/
    views/
    packages/
    settings.py
    transpile.py
    serve.py
  """


def page():
  """
  Create a new page in the current project

  basically it can be something like the below:

  def page():
    '''
    '''

  """


def transpile():
  """
  Transpile to HTML a report
  """
